/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) 2011  <copyright holder> <email>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef TESTSTATETYPE_H
#define TESTSTATETYPE_H

#include <exception>
#include <boost/functional/hash.hpp>

class TestStateType
{
public:
    typedef enum {A = 0, B = 1, C = 2, D = 3} STATE_ID;

private:
  STATE_ID state;

public:
    TestStateType(STATE_ID sid = A) : state(sid) {}

    bool operator==(const TestStateType& other) const
    {
      return state == other.state;
    }
    
    template <typename OutSetType>
    void transition(const char inputSymbol, OutSetType & outSet) const;
    
    bool isHalting() const
    {
      return state == D;
    }
    
    STATE_ID getStateId() const
    {
      return state;
    }
};

template <typename OutSetType>
void TestStateType::transition(const char inputSymbol, OutSetType & outSet) const
{
  switch (state) {
    case A:
      switch (inputSymbol) {
	case 'a':
	  outSet.insert(A);
	  outSet.insert(B);
	  break;
	case 'b':
	  outSet.insert(A);
	  break;
      }
      break;
    case B:
      switch (inputSymbol) {
	case 'b':
	  outSet.insert(C);
	  break;
      }
      break;
    case C:
      switch (inputSymbol) {
	case 'b':
	  outSet.insert(D);
	  break;
      }
      break;
    case D:
      break;
    default:
      throw std::exception();
  }
}

std::size_t hash_value(const TestStateType & s)
{
  return boost::hash_value(s.getStateId());
}

#endif // TESTSTATETYPE_H
